@@ -65,10 +65,11 @@ from bloom.logging import info
 from bloom.logging import sanitize
 from bloom.logging import warning
 
+from bloom.packages import get_package_data
+
 import bloom.util
 from bloom.util import add_global_arguments
 from bloom.util import change_directory
-from bloom.util import get_package_data
 from bloom.util import handle_global_arguments
 
 try:
@@ -114,14 +115,11 @@ def get_upstream_meta(upstream_dir, ros_distro):
             current_branch = get_current_branch()
         else:
             current_branch = None
-        name, version, stackages = get_package_data(
-            current_branch,
-            quiet=False,
-            fuerte=(ros_distro == 'fuerte'))
+        name, version, packages = get_package_data(current_branch, quiet=False)
     meta = {
         'name': name,
         'version': version,
-        'type': 'package.xml' if isinstance(stackages, dict) else 'stack.xml'
+        'type': 'package.xml'
     }
     return meta
 
@@ -136,7 +134,7 @@ def find_version_from_upstream(vcs_uri, vcs_type, devel_branch=None, ros_distro=
     #         return version, None
     #     warning("  Failed to find the version using raw.github.com.")
     # Try to clone the upstream repository
-    info("Checking upstream devel branch for a package.xml(s) or stack.xml")
+    info("Checking upstream devel branch for package.xml(s)")
     upstream_repo = get_upstream_repo(vcs_uri, vcs_type)
     if not upstream_repo.checkout(vcs_uri, devel_branch or ''):
         error("Failed to checkout to the upstream branch "
@@ -144,7 +142,7 @@ def find_version_from_upstream(vcs_uri, vcs_type, devel_branch=None, ros_distro=
               .format(devel_branch or '<default>', vcs_uri), exit=True)
     meta = get_upstream_meta(upstream_repo.get_path(), ros_distro)
     if not meta:
-        error("Failed to find any package.xml(s) or a stack.xml in the "
+        error("Failed to find any package.xml(s) in the "
               "upstream devel branch '{0}' in the repository from '{1}'"
               .format(devel_branch or '<default>', vcs_uri))
     info("Detected version '{0}' from package(s): {1}"
@@ -230,6 +228,14 @@ def process_track_settings(track_dict, release_inc_override):
     return settings
 
 
+def find_full_path(cmd):
+    for path in os.environ.get('PATH', '').split(os.pathsep):
+        full_path = os.path.join(path, cmd)
+        if os.path.isfile(full_path):
+            return full_path
+    raise OSError("[Errno 2] No such file or directory")
+
+
 def execute_track(track, track_dict, release_inc, pretend=True, debug=False, fast=False):
     info("Processing release track settings for '{0}'".format(track))
     settings = process_track_settings(track_dict, release_inc)
@@ -245,6 +251,10 @@ def execute_track(track, track_dict, release_inc, pretend=True, debug=False, fas
     info("", use_prefix=False)
     info("Executing release track '{0}'".format(track))
     for action in track_dict['actions']:
+        if 'bloom-export-upstream' in action and settings['vcs_type'] == 'tar':
+            warning("Explicitly skipping bloom-export-upstream for tar.")
+            settings['archive_path'] = settings['vcs_uri']
+            continue
         templated_action = template_str(action, settings)
         info(fmt("@{bf}@!==> @|@!" + sanitize(str(templated_action))))
         if pretend:
@@ -258,8 +268,10 @@ def execute_track(track, track_dict, release_inc, pretend=True, debug=False, fas
             os.environ['DEBUG'] = '1'
         if fast and 'BLOOM_UNSAFE' not in os.environ:
             os.environ['BLOOM_UNSAFE'] = '1'
+        templated_action = templated_action.split()
+        templated_action[0] = find_full_path(templated_action[0])
         p = subprocess.Popen(templated_action, stdout=stdout, stderr=stderr,
-                             shell=True, env=os.environ.copy())
+                             shell=False, env=os.environ.copy())
         out, err = p.communicate()
         if bloom.util._quiet:
             info(out, use_prefix=False)
@@ -306,6 +318,8 @@ def main(sysargs=None):
     args = parser.parse_args(sysargs)
     handle_global_arguments(args)
 
+    os.environ['BLOOM_TRACK'] = args.track
+
     verify_track(args.track, tracks_dict['tracks'][args.track])
 
     execute_track(args.track, tracks_dict['tracks'][args.track],
@@ -320,4 +334,4 @@ def main(sysargs=None):
          "then push to the release repo with:@|"))
     info(fmt("  git push --all && git push --tags  "
              "@{kf}@!# You might have to add --force to the second command if you "
-             "are over-writing existing flags"))
+             "are over-writing existing tags"))
