@@ -32,12 +32,18 @@
 
 from __future__ import print_function
 
-import yaml
+import os
+import shutil
 import string
+import yaml
+
+from tempfile import mkdtemp
 
-from bloom.git import inbranch
 from bloom.git import branch_exists
 from bloom.git import create_branch
+from bloom.git import has_changes
+from bloom.git import get_root
+from bloom.git import inbranch
 from bloom.git import show
 
 from bloom.logging import error
@@ -46,6 +52,10 @@ from bloom.logging import info
 from bloom.logging import sanitize
 
 from bloom.util import execute_command
+from bloom.util import my_copytree
+
+BLOOM_CONFIG_BRANCH = 'master'
+PLACEHOLDER_FILE = 'CONTENT_MOVED_TO_{0}_BRANCH'.format(BLOOM_CONFIG_BRANCH.upper())
 
 config_spec = {
     'name': {
@@ -111,7 +121,20 @@ This can be any valid relative path in the bloom branch. The contents
 of this folder will be overlaid onto the upstream branch after each
 import-upstream.  Additionally, any package.xml files found in the
 overlay will have the :{version} string replaced with the current
-version being released.'''
+version being released.''',
+        ':{none}': '''\
+Use this if you want to disable overlaying of files.'''
+    },
+    'release_repo_url': {
+        '<url>': '''\
+(optional) Used when pushing to remote release repositories. This is only
+needed when the release uri which is in the rosdistro file is not writable.
+This is useful, for example, when a releaser would like to use a ssh url
+to push rather than a https:// url.
+''',
+    ':{none}': '''\
+This indicates that the default release url should be used.
+'''
     }
 }
 
@@ -161,6 +184,7 @@ DEFAULT_TEMPLATE = {
     'devel_branch': PromptEntry('Upstream Devel Branch', spec=config_spec['devel_branch']),
     'patches': PromptEntry('Patches Directory', spec=config_spec['patches']),
     'ros_distro': PromptEntry('ROS Distro', default='groovy', spec=config_spec['ros_distro']),
+    'release_repo_url': PromptEntry('Release Repository Push URL', spec=config_spec['release_repo_url']),
     'release_inc': -1,
     'actions': [
         'bloom-export-upstream :{vcs_local_uri} :{vcs_type}'
@@ -187,11 +211,12 @@ config_template = {
 
 
 def verify_track(track_name, track):
+    upconvert_bloom_to_config_branch()
     for entry in DEFAULT_TEMPLATE:
         if entry not in track:
             error("Track '{0}' is missing configuration ".format(track_name) +
-                  "'{1}', it may be out of date, please run 'git-bloom-config edit {0}'."
-                  .format(entry), exit=True)
+                  "'{0}', it may be out of date, please run 'git-bloom-config edit {1}'."
+                  .format(entry, track_name), exit=True)
 
 
 class ConfigTemplate(string.Template):
@@ -204,8 +229,9 @@ def template_str(line, settings):
 
 
 def write_tracks_dict_raw(tracks_dict, cmt_msg=None, directory=None):
+    upconvert_bloom_to_config_branch()
     cmt_msg = cmt_msg if cmt_msg is not None else 'Modified tracks.yaml'
-    with inbranch('bloom'):
+    with inbranch(BLOOM_CONFIG_BRANCH):
         with open('tracks.yaml', 'w') as f:
             f.write(yaml.dump(tracks_dict, indent=2, default_flow_style=False))
         execute_command('git add tracks.yaml', cwd=directory)
@@ -214,17 +240,73 @@ def write_tracks_dict_raw(tracks_dict, cmt_msg=None, directory=None):
 
 
 def get_tracks_dict_raw(directory=None):
-    if not branch_exists('bloom'):
-        info("Creating bloom branch.")
-        create_branch('bloom', orphaned=True, directory=directory)
-    tracks_yaml = show('bloom', 'tracks.yaml', directory=directory)
+    upconvert_bloom_to_config_branch()
+    if not branch_exists(BLOOM_CONFIG_BRANCH):
+        info("Creating '{0}' branch.".format(BLOOM_CONFIG_BRANCH))
+        create_branch(BLOOM_CONFIG_BRANCH, orphaned=True, directory=directory)
+    tracks_yaml = show(BLOOM_CONFIG_BRANCH, 'tracks.yaml', directory=directory)
     if not tracks_yaml:
         write_tracks_dict_raw(
             {'tracks': {}}, 'Initial tracks.yaml', directory=directory
         )
-        tracks_yaml = show('bloom', 'tracks.yaml', directory=directory)
+        tracks_yaml = show(BLOOM_CONFIG_BRANCH, 'tracks.yaml',
+                           directory=directory)
     try:
         return yaml.load(tracks_yaml)
     except:
         # TODO handle yaml errors
         raise
+
+_has_checked_bloom_branch = False
+
+
+def upconvert_bloom_to_config_branch():
+    global _has_checked_bloom_branch
+    if _has_checked_bloom_branch:
+        return
+    if get_root() is None:
+        # Not a git repository
+        return
+    if show('bloom', PLACEHOLDER_FILE) is not None:
+        return
+    if show('bloom', 'bloom.conf') is not None:
+        # Wait for the bloom.conf upconvert...
+        return
+    if not branch_exists('bloom'):
+        return
+    _has_checked_bloom_branch = True
+    info("Moving configurations from deprecated 'bloom' branch "
+         "to the '{0}' branch.".format(BLOOM_CONFIG_BRANCH))
+    tmp_dir = mkdtemp()
+    git_root = get_root()
+    try:
+        # Copy the new upstream source into the temporary directory
+        with inbranch('bloom'):
+            ignores = ('.git', '.gitignore', '.svn', '.hgignore', '.hg', 'CVS')
+            configs = os.path.join(tmp_dir, 'configs')
+            my_copytree(git_root, configs, ignores)
+            execute_command('git rm -rf ./*')
+            with open(PLACEHOLDER_FILE, 'w') as f:
+                f.write("""\
+This branch ('bloom') has been deprecated in favor of storing settings and overlay files in the master branch.
+
+Please goto the master branch for anything which referenced the bloom branch.
+
+You can delete this branch at your convenience.
+""")
+            execute_command('git add ' + PLACEHOLDER_FILE)
+            if has_changes():
+                execute_command('git commit -m "DEPRECATING BRANCH"')
+        if not branch_exists(BLOOM_CONFIG_BRANCH):
+            info("Creating '{0}' branch.".format(BLOOM_CONFIG_BRANCH))
+            create_branch(BLOOM_CONFIG_BRANCH, orphaned=True)
+        with inbranch(BLOOM_CONFIG_BRANCH):
+            my_copytree(configs, git_root)
+            execute_command('git add ./*')
+            if has_changes():
+                execute_command('git commit -m '
+                                '"Moving configs from bloom branch"')
+    finally:
+        # Clean up
+        if os.path.exists(tmp_dir):
+            shutil.rmtree(tmp_dir)
