@@ -39,9 +39,13 @@ from __future__ import print_function
 import argparse
 import os
 import shutil
+import socket
 import sys
 import tempfile
-import traceback
+import time
+import urllib2
+
+from email.utils import formatdate
 
 from subprocess import CalledProcessError
 from subprocess import PIPE
@@ -55,17 +59,8 @@ from bloom.logging import debug
 from bloom.logging import disable_ANSI_colors
 from bloom.logging import enable_debug
 from bloom.logging import error
-from bloom.logging import info
 from bloom.logging import warning
 
-try:
-    from catkin_pkg.packages import find_packages
-    from catkin_pkg.packages import verify_equal_package_versions
-except ImportError:
-    debug(traceback.format_exc())
-    error("catkin_pkg was not detected, please install it.",
-          file=sys.stderr, exit=True)
-
 
 # Convention: < 0 is a warning exit, 0 is normal, > 0 is an error
 class code(object):
@@ -141,56 +136,67 @@ class temporary_directory(object):
             os.chdir(self.original_cwd)
 
 
-def get_package_data(branch_name=None, directory=None, quiet=True, fuerte=False):
-    """
-    Gets package data about the package(s) in the current branch.
+def get_rfc_2822_date(date):
+    return formatdate(float(date.strftime("%s")), date.tzinfo)
 
-    :param branch_name: name of the branch you are searching on (log use only)
+
+def load_url_to_file_handle(url, retry=2, retry_period=1, timeout=10):
+    """Loads a given url with retries, retry_periods, and timeouts
+
+    Based on https://github.com/ros-infrastructure/rosdistro/blob/master/src/rosdistro/loader.py
+
+    :param url: URL to load and return contents of
+    :type url: str
+    :param retry: number of times to retry the url on 503 or timeout
+    :type retry: int
+    :param retry_period: time to wait between retries in seconds
+    :type: retry_period: float
+    :param timeout: timeout for opening the URL in seconds
+    :type timeout: float
     """
-    log = debug if quiet else info
-    repo_dir = directory if directory else os.getcwd()
-    stack_path = os.path.join(repo_dir, 'stack.xml')
-    if os.path.exists(stack_path) and not fuerte:
-            warning("stack.xml is present but going to be ignored because this is not a release for Fuerte.")
-    if branch_name:
-        log("Looking for packages in '{0}' branch... ".format(branch_name), end='')
-    else:
-        log("Looking for packages in '{0}'... ".format(directory or os.getcwd()), end='')
-    ## Check for package.xml(s)
-    if not fuerte:
-        packages = find_packages(repo_dir)
-    else:
-        packages = {}
-    if type(packages) == dict and packages != {}:
-        if len(packages) > 1:
-            log("found " + str(len(packages)) + " packages.",
-                use_prefix=False)
-        else:
-            log("found '" + packages.values()[0].name + "'.",
-                use_prefix=False)
-        version = verify_equal_package_versions(packages.values())
-        return [p.name for p in packages.values()], version, packages
-    ## Check for stack.xml
-    has_rospkg = False
     try:
-        import rospkg
-        has_rospkg = True
-    except ImportError:
-        log(ansi('redf') + "failed." + ansi('reset'), use_prefix=False)
-        warning("rospkg was not detected, stack.xml discovery is disabled",
-                file=sys.stderr)
-    if not has_rospkg:
-        error("no package.xml(s) found, and no name specified with "
-              "'--package-name', aborting.", use_prefix=False, exit=True)
-    if os.path.exists(stack_path):
-        log("found stack.xml.", use_prefix=False)
-        stack = rospkg.stack.parse_stack_file(stack_path)
-        return stack.name, stack.version, stack
-    # Otherwise we have a problem
-    log("failed.", use_prefix=False)
-    error("no package.xml(s) or stack.xml found, and no name "
-          "specified with '--package-name', aborting.",
-          use_prefix=False, exit=True)
+        fh = urllib2.urlopen(url, timeout=timeout)
+    except urllib2.HTTPError as e:
+        if e.code == 503 and retry:
+            time.sleep(retry_period)
+            return load_url_to_file_handle(url,
+                                           retry=retry - 1,
+                                           retry_period=retry_period,
+                                           timeout=timeout)
+        e.msg += ' (%s)' % url
+        raise
+    except urllib2.URLError as e:
+        if isinstance(e.reason, socket.timeout) and retry:
+            time.sleep(retry_period)
+            return load_url_to_file_handle(url,
+                                           retry=retry - 1,
+                                           retry_period=retry_period,
+                                           timeout=timeout)
+        raise urllib2.URLError(str(e) + ' (%s)' % url)
+    return fh
+
+
+def my_copytree(tree, destination, ignores=None):
+    ignores = ignores or []
+    if os.path.exists(destination):
+        if not os.path.isdir(destination):
+            raise RuntimeError("Destination exists and is not a directory: '{0}'".format(destination))
+    else:
+        os.makedirs(destination)
+    for item in os.listdir(tree):
+        if item in ignores:
+            continue
+        src = os.path.join(tree, item)
+        dst = os.path.join(destination, item)
+        if os.path.islink(src):
+            linkto = os.readlink(src)
+            os.symlink(linkto, dst)
+        elif os.path.isdir(src):
+            my_copytree(src, dst, ignores)
+        elif os.path.isfile(src):
+            shutil.copy(src, dst)
+        else:
+            raise RuntimeError("Unknown file type for element: '{0}'".format(src))
 
 
 def add_global_arguments(parser):
@@ -219,6 +225,7 @@ _disable_git_clone = False
 def disable_git_clone(state=True):
     global _disable_git_clone
     _disable_git_clone = state
+    os.environ['BLOOM_UNSAFE'] = "1"
 
 
 def get_git_clone_state():
@@ -328,7 +335,7 @@ def extract_text(element):
 def segment_version(full_version):
     version_list = full_version.split('.')
     if len(version_list) != 3:
-        warning('Invalid version element in the stack.xml, expected: '
+        warning('Invalid version element, expected: '
                 '<major>.<minor>.<patch>')
     if len(version_list) < 3:
         sys.exit(code.INVALID_VERSION)
